@@ -10,6 +10,8 @@ import tensorflow as tf
 from poros.poros_dataset import about_tensor
 from poros.poros_train import acitvation_function
 from tensorflow.python.framework import tensor_shape
+from tensorflow.keras import activations, initializers, regularizers, constraints
+from tensorflow.keras.layers import InputSpec
 import numpy as np
 
 
@@ -461,6 +463,12 @@ class TransformerLayer(tf.keras.layers.Layer):
                             kernel_initializer=create_initializer(self.initializer_range))
                         layer.build(input_shape=[None, self.intermediate_size])
                         self.outputs.append(layer)
+
+                    with tf.name_scope("LayerNorm"):
+                        layer = tf.keras.layers.LayerNormalization(epsilon=0.00001)
+                        layer.build(input_shape=[None, None, self.hidden_size])
+                        self.outputs_layer_norm.append(layer)
+
                     with tf.name_scope("rezero"):
                         layer = RezeroLayer()
                         self.dense_output_rezero_layers.append(layer)
@@ -488,14 +496,16 @@ class TransformerLayer(tf.keras.layers.Layer):
         all_layer_outputs = []
 
         for (attention_layer, attention_output_layer, intermediate_output, output,
-             attention_outputs_rezero_layer, dense_output_rezero_layer,attention_outputs_layer_norm) \
+             attention_outputs_rezero_layer, dense_output_rezero_layer,attention_outputs_layer_norm,
+             outputs_layer_norm) \
                 in zip(self.attention_layers,
                        self.attention_outputs,
                        self.intermediate_outputs,
                        self.outputs,
                        self.attention_outputs_rezero_layers,
                        self.dense_output_rezero_layers,
-                       self.attention_outputs_layer_norm
+                       self.attention_outputs_layer_norm,
+                       self.outputs_layer_norm
                        ):
             layer_input = prev_output
             attention_heads = []
@@ -514,7 +524,7 @@ class TransformerLayer(tf.keras.layers.Layer):
             if training:
                 attention_output = dropout(attention_output, self.hidden_dropout_prob)
 
-            attention_output = attention_output + attention_outputs_rezero_layer(attention_output)
+            attention_output = attention_outputs_layer_norm(layer_input+attention_output)
             if training:
                 attention_output = dropout(attention_output, self.hidden_dropout_prob)
 
@@ -525,7 +535,7 @@ class TransformerLayer(tf.keras.layers.Layer):
             layer_output = output(intermediate_output)
             if training:
                 layer_output = dropout(layer_output, self.hidden_dropout_prob)
-            layer_output = attention_output + dense_output_rezero_layer(layer_output)
+            layer_output = outputs_layer_norm(layer_output + attention_output)
             if training:
                 layer_output = dropout(layer_output, self.hidden_dropout_prob)
             prev_output = layer_output
@@ -671,7 +681,7 @@ class RezeroLayer(tf.keras.layers.Layer):
         tensor_shape.TensorShape(input_shape)
         shape = [tensor_shape.dimension_value(input_shape[-2]), tensor_shape.dimension_value(input_shape[-1])]
         self.alpha = self.add_weight(name="alpha",
-                        shape=shape,
+                        shape=[1],
                         initializer=tf.initializers.Zeros,
                         trainable=True)
         self.built = True
@@ -679,3 +689,230 @@ class RezeroLayer(tf.keras.layers.Layer):
     def call(self, inputs):
         outputs = tf.multiply(self.alpha, inputs)
         return outputs
+    
+
+class MMoELayer(tf.keras.layers.Layer):
+    """
+    参数为专家个数
+
+    """
+
+    def __init__(self,
+                 units,
+                 num_experts,
+                 num_tasks,
+                 use_expert_bias=True,
+                 use_gate_bias=True,
+                 expert_activation='relu',
+                 gate_activation='softmax',
+                 expert_bias_initializer='zeros',
+                 gate_bias_initializer='zeros',
+                 expert_bias_regularizer=None,
+                 gate_bias_regularizer=None,
+                 expert_bias_constraint=None,
+                 gate_bias_constraint=None,
+                 expert_kernel_initializer='VarianceScaling',
+                 gate_kernel_initializer='VarianceScaling',
+                 expert_kernel_regularizer=None,
+                 gate_kernel_regularizer=None,
+                 expert_kernel_constraint=None,
+                 gate_kernel_constraint=None,
+                 activity_regularizer=None,
+                 **kwargs):
+        """
+         Method for instantiating MMoE layer.
+        :param units: Number of hidden units
+        :param num_experts: Number of experts
+        :param num_tasks: Number of tasks
+        :param use_expert_bias: Boolean to indicate the usage of bias in the expert weights
+        :param use_gate_bias: Boolean to indicate the usage of bias in the gate weights
+        :param expert_activation: Activation function of the expert weights
+        :param gate_activation: Activation function of the gate weights
+        :param expert_bias_initializer: Initializer for the expert bias
+        :param gate_bias_initializer: Initializer for the gate bias
+        :param expert_bias_regularizer: Regularizer for the expert bias
+        :param gate_bias_regularizer: Regularizer for the gate bias
+        :param expert_bias_constraint: Constraint for the expert bias
+        :param gate_bias_constraint: Constraint for the gate bias
+        :param expert_kernel_initializer: Initializer for the expert weights
+        :param gate_kernel_initializer: Initializer for the gate weights
+        :param expert_kernel_regularizer: Regularizer for the expert weights
+        :param gate_kernel_regularizer: Regularizer for the gate weights
+        :param expert_kernel_constraint: Constraint for the expert weights
+        :param gate_kernel_constraint: Constraint for the gate weights
+        :param activity_regularizer: Regularizer for the activity
+        :param kwargs: Additional keyword arguments for the Layer class
+        """
+        # Hidden nodes parameter
+        self.units = units
+        self.num_experts = num_experts
+        self.num_tasks = num_tasks
+
+        # Weight parameter
+        self.expert_kernels = None
+        self.gate_kernels = None
+        self.expert_kernel_initializer = initializers.get(expert_kernel_initializer)
+        self.gate_kernel_initializer = initializers.get(gate_kernel_initializer)
+        self.expert_kernel_regularizer = regularizers.get(expert_kernel_regularizer)
+        self.gate_kernel_regularizer = regularizers.get(gate_kernel_regularizer)
+        self.expert_kernel_constraint = constraints.get(expert_kernel_constraint)
+        self.gate_kernel_constraint = constraints.get(gate_kernel_constraint)
+
+        # Activation parameter
+        self.expert_activation = activations.get(expert_activation)
+        self.gate_activation = activations.get(gate_activation)
+
+        # Bias parameter
+        self.expert_bias = None
+        self.gate_bias = None
+        self.use_expert_bias = use_expert_bias
+        self.use_gate_bias = use_gate_bias
+        self.expert_bias_initializer = initializers.get(expert_bias_initializer)
+        self.gate_bias_initializer = initializers.get(gate_bias_initializer)
+        self.expert_bias_regularizer = regularizers.get(expert_bias_regularizer)
+        self.gate_bias_regularizer = regularizers.get(gate_bias_regularizer)
+        self.expert_bias_constraint = constraints.get(expert_bias_constraint)
+        self.gate_bias_constraint = constraints.get(gate_bias_constraint)
+
+        # Activity parameter
+        self.activity_regularizer = regularizers.get(activity_regularizer)
+
+        # Keras parameter
+        self.input_spec = InputSpec(min_ndim=2)
+        self.supports_masking = True
+
+        super(MMoELayer, self).__init__(**kwargs)
+
+    def build(self, input_shape):
+        """
+        Method for creating the layer weights.
+        :param input_shape: Keras tensor (future input to layer)
+                            or list/tuple of Keras tensors to reference
+                            for weight shape computations
+        """
+        assert input_shape is not None and len(input_shape) >= 2
+
+        input_dimension = input_shape[-1]
+
+        # Initialize expert weights (number of input features * number of units per expert * number of experts)
+        self.expert_kernels = self.add_weight(
+            name='expert_kernel',
+            shape=(input_dimension, self.units, self.num_experts),
+            initializer=self.expert_kernel_initializer,
+            regularizer=self.expert_kernel_regularizer,
+            constraint=self.expert_kernel_constraint,
+        )
+
+        # Initialize expert bias (number of units per expert * number of experts)
+        if self.use_expert_bias:
+            self.expert_bias = self.add_weight(
+                name='expert_bias',
+                shape=(self.units, self.num_experts),
+                initializer=self.expert_bias_initializer,
+                regularizer=self.expert_bias_regularizer,
+                constraint=self.expert_bias_constraint,
+            )
+
+        # Initialize gate weights (number of input features * number of experts * number of tasks)
+        self.gate_kernels = [self.add_weight(
+            name='gate_kernel_task_{}'.format(i),
+            shape=(input_dimension, self.num_experts),
+            initializer=self.gate_kernel_initializer,
+            regularizer=self.gate_kernel_regularizer,
+            constraint=self.gate_kernel_constraint
+        ) for i in range(self.num_tasks)]
+
+        # Initialize gate bias (number of experts * number of tasks)
+        if self.use_gate_bias:
+            self.gate_bias = [self.add_weight(
+                name='gate_bias_task_{}'.format(i),
+                shape=(self.num_experts,),
+                initializer=self.gate_bias_initializer,
+                regularizer=self.gate_bias_regularizer,
+                constraint=self.gate_bias_constraint
+            ) for i in range(self.num_tasks)]
+
+        self.input_spec = InputSpec(min_ndim=2, axes={-1: input_dimension})
+
+        super(MMoELayer, self).build(input_shape)
+
+    def call(self, inputs, **kwargs):
+        """
+        Method for the forward function of the layer.
+        :param inputs: Input tensor
+        :param kwargs: Additional keyword arguments for the base method
+        :return: A tensor
+        """
+        gate_outputs = []
+        final_outputs = []
+
+        # f_{i}(x) = activation(W_{i} * x + b), where activation is ReLU according to the paper
+        expert_outputs = tf.tensordot(a=inputs, b=self.expert_kernels, axes=1)
+        # Add the bias term to the expert weights if necessary
+        if self.use_expert_bias:
+            expert_outputs = tf.keras.backend.bias_add(x=expert_outputs, bias=self.expert_bias)
+        expert_outputs = self.expert_activation(expert_outputs)
+
+        # g^{k}(x) = activation(W_{gk} * x + b), where activation is softmax according to the paper
+        for index, gate_kernel in enumerate(self.gate_kernels):
+            gate_output = tf.keras.backend.dot(x=inputs, y=gate_kernel)
+            # Add the bias term to the gate weights if necessary
+            if self.use_gate_bias:
+                gate_output = tf.keras.backend.bias_add(x=gate_output, bias=self.gate_bias[index])
+            gate_output = self.gate_activation(gate_output)
+            gate_outputs.append(gate_output)
+
+        # f^{k}(x) = sum_{i=1}^{n}(g^{k}(x)_{i} * f_{i}(x))
+        for gate_output in gate_outputs:
+            expanded_gate_output = tf.expand_dims(gate_output, axis=1)
+            weighted_expert_output = expert_outputs * tf.keras.backend.repeat_elements(expanded_gate_output, self.units, axis=1)
+            final_outputs.append(tf.keras.backend.sum(weighted_expert_output, axis=2))
+
+        return final_outputs
+
+    def compute_output_shape(self, input_shape):
+        """
+        Method for computing the output shape of the MMoE layer.
+        :param input_shape: Shape tuple (tuple of integers)
+        :return: List of input shape tuple where the size of the list is equal to the number of tasks
+        """
+        assert input_shape is not None and len(input_shape) >= 2
+
+        output_shape = list(input_shape)
+        output_shape[-1] = self.units
+        output_shape = tuple(output_shape)
+
+        return [output_shape for _ in range(self.num_tasks)]
+
+    def get_config(self):
+        """
+        Method for returning the configuration of the MMoE layer.
+        :return: Config dictionary
+        """
+
+        config = {
+            'units': self.units,
+            'num_experts': self.num_experts,
+            'num_tasks': self.num_tasks,
+            'use_expert_bias': self.use_expert_bias,
+            'use_gate_bias': self.use_gate_bias,
+            'expert_activation': activations.serialize(self.expert_activation),
+            'gate_activation': activations.serialize(self.gate_activation),
+            'expert_bias_initializer': initializers.serialize(self.expert_bias_initializer),
+            'gate_bias_initializer': initializers.serialize(self.gate_bias_initializer),
+            'expert_bias_regularizer': regularizers.serialize(self.expert_bias_regularizer),
+            'gate_bias_regularizer': regularizers.serialize(self.gate_bias_regularizer),
+            'expert_bias_constraint': constraints.serialize(self.expert_bias_constraint),
+            'gate_bias_constraint': constraints.serialize(self.gate_bias_constraint),
+            'expert_kernel_initializer': initializers.serialize(self.expert_kernel_initializer),
+            'gate_kernel_initializer': initializers.serialize(self.gate_kernel_initializer),
+            'expert_kernel_regularizer': regularizers.serialize(self.expert_kernel_regularizer),
+            'gate_kernel_regularizer': regularizers.serialize(self.gate_kernel_regularizer),
+            'expert_kernel_constraint': constraints.serialize(self.expert_kernel_constraint),
+            'gate_kernel_constraint': constraints.serialize(self.gate_kernel_constraint),
+            'activity_regularizer': regularizers.serialize(self.activity_regularizer)
+        }
+        base_config = super(MMoELayer, self).get_config()
+
+        return dict(list(base_config.items()) + list(config.items()))
+
